@@ -17,10 +17,17 @@ from six.moves import queue
 from ray.util.debug import log_once
 from ray.tune import TuneError, session
 from ray.tune.trainable import Trainable, TrainableUtil
-from ray.tune.result import (DEFAULT_METRIC, TIME_THIS_ITER_S,
-                             RESULT_DUPLICATE, SHOULD_CHECKPOINT)
-from ray.tune.utils import (detect_checkpoint_function, detect_config_single,
-                            detect_reporter)
+from ray.tune.result import (
+    DEFAULT_METRIC,
+    TIME_THIS_ITER_S,
+    RESULT_DUPLICATE,
+    SHOULD_CHECKPOINT,
+)
+from ray.tune.utils import (
+    detect_checkpoint_function,
+    detect_config_single,
+    detect_reporter,
+)
 from ray.tune.utils.trainable import with_parameters  # noqa: F401
 
 logger = logging.getLogger(__name__)
@@ -71,7 +78,8 @@ class FuncCheckpointUtil:
     def mk_null_checkpoint_dir(logdir):
         """Indicate that the given checkpoint doesn't have state."""
         checkpoint_dir = TrainableUtil.make_checkpoint_dir(
-            logdir, index=-1, override=True)
+            logdir, index=-1, override=True
+        )
         open(os.path.join(checkpoint_dir, NULL_MARKER), "a").close()
         return checkpoint_dir
 
@@ -79,7 +87,8 @@ class FuncCheckpointUtil:
     def mk_temp_checkpoint_dir(logdir):
         """Indicate that the checkpoint is only for restoration."""
         temporary_checkpoint_dir = TrainableUtil.make_checkpoint_dir(
-            logdir, index="tmp" + uuid.uuid4().hex[:6], override=True)
+            logdir, index="tmp" + uuid.uuid4().hex[:6], override=True
+        )
         open(os.path.join(temporary_checkpoint_dir, TEMP_MARKER), "a").close()
         return temporary_checkpoint_dir
 
@@ -98,16 +107,17 @@ class FuncCheckpointUtil:
         """Copies temporary checkpoint to a permanent checkpoint directory."""
         checkpoint_dir = os.path.abspath(checkpoint_dir)
         temporary_marker = os.path.join(checkpoint_dir, TEMP_MARKER)
-        assert os.path.exists(temporary_marker), (
-            "Should not be calling this method on a permanent checkpoint.")
+        assert os.path.exists(
+            temporary_marker
+        ), "Should not be calling this method on a permanent checkpoint."
         os.remove(temporary_marker)
         perm_checkpoint_dir = TrainableUtil.make_checkpoint_dir(
-            logdir, index=step, override=True)
+            logdir, index=step, override=True
+        )
         shutil.rmtree(perm_checkpoint_dir)
 
         shutil.copytree(checkpoint_dir, perm_checkpoint_dir)
-        assert not os.path.exists(
-            os.path.join(perm_checkpoint_dir, TEMP_MARKER))
+        assert not os.path.exists(os.path.join(perm_checkpoint_dir, TEMP_MARKER))
         return perm_checkpoint_dir
 
 
@@ -120,14 +130,16 @@ class StatusReporter:
         >>>     reporter(timesteps_this_iter=1)
     """
 
-    def __init__(self,
-                 result_queue,
-                 continue_semaphore,
-                 end_event,
-                 trial_name=None,
-                 trial_id=None,
-                 logdir=None,
-                 trial_resources=None):
+    def __init__(
+        self,
+        result_queue,
+        continue_semaphore,
+        end_event,
+        trial_name=None,
+        trial_id=None,
+        logdir=None,
+        trial_resources=None,
+    ):
         self._queue = result_queue
         self._last_report_time = None
         self._continue_semaphore = continue_semaphore
@@ -139,11 +151,7 @@ class StatusReporter:
         self._fresh_checkpoint = False
         self._trial_resources = trial_resources
 
-    def reset(self,
-              trial_name=None,
-              trial_id=None,
-              logdir=None,
-              trial_resources=None):
+    def reset(self, trial_name=None, trial_id=None, logdir=None, trial_resources=None):
         self._trial_name = trial_name
         self._trial_id = trial_id
         self._logdir = logdir
@@ -170,7 +178,8 @@ class StatusReporter:
 
         assert self._last_report_time is not None, (
             "StatusReporter._start() must be called before the first "
-            "report __call__ is made to ensure correct runtime metrics.")
+            "report __call__ is made to ensure correct runtime metrics."
+        )
 
         if _metric:
             kwargs[DEFAULT_METRIC] = _metric
@@ -196,8 +205,7 @@ class StatusReporter:
             sys.exit(0)
 
     def make_checkpoint_dir(self, step):
-        checkpoint_dir = TrainableUtil.make_checkpoint_dir(
-            self.logdir, index=step)
+        checkpoint_dir = TrainableUtil.make_checkpoint_dir(self.logdir, index=step)
         logger.debug("Making checkpoint dir at %s", checkpoint_dir)
         return checkpoint_dir
 
@@ -211,8 +219,10 @@ class StatusReporter:
             try:
                 TrainableUtil.find_checkpoint_dir(checkpoint)
             except FileNotFoundError:
-                logger.error("Checkpoint must be created with path given from "
-                             "make_checkpoint_dir.")
+                logger.error(
+                    "Checkpoint must be created with path given from "
+                    "make_checkpoint_dir."
+                )
                 raise
         self._last_checkpoint = checkpoint
         if is_new:
@@ -262,8 +272,11 @@ class _RunnerThread(threading.Thread):
             self._entrypoint()
         except StopIteration:
             logger.debug(
-                ("Thread runner raised StopIteration. Interperting it as a "
-                 "signal to terminate the thread without error."))
+                (
+                    "Thread runner raised StopIteration. Interperting it as a "
+                    "signal to terminate the thread without error."
+                )
+            )
         except Exception as e:
             logger.exception("Runner Thread raised error.")
             try:
@@ -272,12 +285,16 @@ class _RunnerThread(threading.Thread):
                 # case that something went terribly wrong
                 err_tb_str = traceback.format_exc()
                 self._error_queue.put(
-                    err_tb_str, block=True, timeout=ERROR_REPORT_TIMEOUT)
+                    err_tb_str, block=True, timeout=ERROR_REPORT_TIMEOUT
+                )
             except queue.Full:
                 logger.critical(
-                    ("Runner Thread was unable to report error to main "
-                     "function runner thread. This means a previous error "
-                     "was not processed. This should never happen."))
+                    (
+                        "Runner Thread was unable to report error to main "
+                        "function runner thread. This means a previous error "
+                        "was not processed. This should never happen."
+                    )
+                )
             raise e
 
 
@@ -312,7 +329,8 @@ class FunctionRunner(Trainable):
             trial_name=self.trial_name,
             trial_id=self.trial_id,
             logdir=self.logdir,
-            trial_resources=self.trial_resources)
+            trial_resources=self.trial_resources,
+        )
         self._last_result = {}
 
         session.init(self._status_reporter)
@@ -327,8 +345,11 @@ class FunctionRunner(Trainable):
 
     def _start(self):
         def entrypoint():
-            return self._trainable_func(self.config, self._status_reporter,
-                                        self._status_reporter.get_checkpoint())
+            return self._trainable_func(
+                self.config,
+                self._status_reporter,
+                self._status_reporter.get_checkpoint(),
+            )
 
         # the runner thread is not started until the first call to _train
         self._runner = _RunnerThread(entrypoint, self._error_queue)
@@ -361,7 +382,8 @@ class FunctionRunner(Trainable):
             # fetch the next produced result
             try:
                 result = self._results_queue.get(
-                    block=True, timeout=RESULT_FETCH_TIMEOUT)
+                    block=True, timeout=RESULT_FETCH_TIMEOUT
+                )
             except queue.Empty:
                 pass
 
@@ -386,14 +408,20 @@ class FunctionRunner(Trainable):
             # runner thread never reported any results which should not be
             # possible when wrapping functions with `wrap_function`.
             raise TuneError(
-                ("Wrapped function ran until completion without reporting "
-                 "results or raising an exception."))
+                (
+                    "Wrapped function ran until completion without reporting "
+                    "results or raising an exception."
+                )
+            )
 
         else:
             if not self._error_queue.empty():
                 logger.warning(
-                    ("Runner error waiting to be raised in main thread. "
-                     "Logging all available results first."))
+                    (
+                        "Runner error waiting to be raised in main thread. "
+                        "Logging all available results first."
+                    )
+                )
 
         # This keyword appears if the train_func using the Function API
         # finishes without "done=True". This duplicates the last result, but
@@ -411,10 +439,9 @@ class FunctionRunner(Trainable):
     def execute(self, fn):
         return fn(self)
 
-    def save(self, checkpoint_path=None):
+    def save(self, checkpoint_path=None) -> str:
         if checkpoint_path:
-            raise ValueError(
-                "Checkpoint path should not be used with function API.")
+            raise ValueError("Checkpoint path should not be used with function API.")
 
         checkpoint = self._status_reporter.get_checkpoint()
         state = self.get_state()
@@ -426,7 +453,8 @@ class FunctionRunner(Trainable):
             parent_dir = checkpoint
         elif isinstance(checkpoint, dict):
             parent_dir = TrainableUtil.make_checkpoint_dir(
-                self.logdir, index=self.training_iteration)
+                self.logdir, index=self.training_iteration
+            )
         elif isinstance(checkpoint, str):
             parent_dir = TrainableUtil.find_checkpoint_dir(checkpoint)
             # When the trainable is restored, a temporary checkpoint
@@ -438,18 +466,22 @@ class FunctionRunner(Trainable):
                 parent_dir = FuncCheckpointUtil.create_perm_checkpoint(
                     checkpoint_dir=parent_dir,
                     logdir=self.logdir,
-                    step=self.training_iteration)
-                checkpoint = os.path.abspath(
-                    os.path.join(parent_dir, relative_path))
+                    step=self.training_iteration,
+                )
+                checkpoint = os.path.abspath(os.path.join(parent_dir, relative_path))
         else:
-            raise ValueError("Provided checkpoint was expected to have "
-                             "type (str, dict). Got {}.".format(
-                                 type(checkpoint)))
+            raise ValueError(
+                "Provided checkpoint was expected to have "
+                "type (str, dict). Got {}.".format(type(checkpoint))
+            )
 
         checkpoint_path = TrainableUtil.process_checkpoint(
-            checkpoint, parent_dir, state)
+            checkpoint, parent_dir, state
+        )
 
-        self._maybe_save_to_cloud()
+        self._postprocess_checkpoint(checkpoint_path)
+
+        self._maybe_save_to_cloud(parent_dir)
 
         return checkpoint_path
 
@@ -472,10 +504,12 @@ class FunctionRunner(Trainable):
         self._status_reporter.set_checkpoint(checkpoint, is_new=False)
 
     def restore_from_object(self, obj):
-        self.temp_checkpoint_dir = (FuncCheckpointUtil.mk_temp_checkpoint_dir(
-            self.logdir))
+        self.temp_checkpoint_dir = FuncCheckpointUtil.mk_temp_checkpoint_dir(
+            self.logdir
+        )
         checkpoint_path = TrainableUtil.create_from_pickle(
-            obj, self.temp_checkpoint_dir)
+            obj, self.temp_checkpoint_dir
+        )
         self.restore(checkpoint_path)
 
     def cleanup(self):
@@ -487,18 +521,21 @@ class FunctionRunner(Trainable):
         # If everything stayed in synch properly, this should never happen.
         if not self._results_queue.empty():
             logger.warning(
-                ("Some results were added after the trial stop condition. "
-                 "These results won't be logged."))
+                (
+                    "Some results were added after the trial stop condition. "
+                    "These results won't be logged."
+                )
+            )
 
         # Check for any errors that might have been missed.
         self._report_thread_runner_error()
         session.shutdown()
 
         if self.temp_checkpoint_dir is not None and os.path.exists(
-                self.temp_checkpoint_dir):
+            self.temp_checkpoint_dir
+        ):
             shutil.rmtree(self.temp_checkpoint_dir)
-            logger.debug("Clearing temporary checkpoint: %s",
-                         self.temp_checkpoint_dir)
+            logger.debug("Clearing temporary checkpoint: %s", self.temp_checkpoint_dir)
 
     def reset_config(self, new_config):
         if self._runner and self._runner.is_alive():
@@ -506,8 +543,7 @@ class FunctionRunner(Trainable):
             self._continue_semaphore.release()
             # Wait for thread termination so it is save to re-use the same
             # actor.
-            thread_timeout = int(
-                os.environ.get("TUNE_FUNCTION_THREAD_TIMEOUT_S", 2))
+            thread_timeout = int(os.environ.get("TUNE_FUNCTION_THREAD_TIMEOUT_S", 2))
             self._runner.join(timeout=thread_timeout)
             if self._runner.is_alive():
                 # Did not finish within timeout, reset unsuccessful.
@@ -520,25 +556,25 @@ class FunctionRunner(Trainable):
             trial_name=self.trial_name,
             trial_id=self.trial_id,
             logdir=self.logdir,
-            trial_resources=self.trial_resources)
+            trial_resources=self.trial_resources,
+        )
 
         return True
 
     def _report_thread_runner_error(self, block=False):
         try:
-            err_tb_str = self._error_queue.get(
-                block=block, timeout=ERROR_FETCH_TIMEOUT)
+            err_tb_str = self._error_queue.get(block=block, timeout=ERROR_FETCH_TIMEOUT)
             raise TuneError(
-                ("Trial raised an exception. Traceback:\n{}".format(err_tb_str)
-                 ))
+                ("Trial raised an exception. Traceback:\n{}".format(err_tb_str))
+            )
         except queue.Empty:
             pass
 
 
-def wrap_function(train_func: Callable[[Any], Any],
-                  warn: bool = True,
-                  name: Optional[str] = None):
-    inherit_from = (FunctionRunner, )
+def wrap_function(
+    train_func: Callable[[Any], Any], warn: bool = True, name: Optional[str] = None
+):
+    inherit_from = (FunctionRunner,)
 
     if hasattr(train_func, "__mixins__"):
         inherit_from = train_func.__mixins__ + inherit_from
@@ -554,7 +590,8 @@ def wrap_function(train_func: Callable[[Any], Any],
             "Unknown argument found in the Trainable function. "
             "The function args must include a 'config' positional "
             "parameter. Any other args must be 'checkpoint_dir'. "
-            "Found: {}".format(func_args))
+            "Found: {}".format(func_args)
+        )
 
     if use_config_single and not use_checkpoint:
         if log_once("tune_function_checkpoint") and warn:
@@ -562,11 +599,13 @@ def wrap_function(train_func: Callable[[Any], Any],
                 "Function checkpointing is disabled. This may result in "
                 "unexpected behavior when using checkpointing features or "
                 "certain schedulers. To enable, set the train function "
-                "arguments to be `func(config, checkpoint_dir=None)`.")
+                "arguments to be `func(config, checkpoint_dir=None)`."
+            )
 
     class ImplicitFunc(*inherit_from):
-        _name = name or (train_func.__name__
-                         if hasattr(train_func, "__name__") else "func")
+        _name = name or (
+            train_func.__name__ if hasattr(train_func, "__name__") else "func"
+        )
 
         def __repr__(self):
             return self._name
@@ -590,7 +629,8 @@ def wrap_function(train_func: Callable[[Any], Any],
                     raise ValueError(
                         "Invalid return or yield value. Either return/yield "
                         "a single number or a dictionary object in your "
-                        "trainable function.")
+                        "trainable function."
+                    )
 
             output = None
             if inspect.isgeneratorfunction(train_func):
